#include "image.h"
#include <bit>
#include <iostream>
//#define WIN32_LEAN_AND_MEAN
#define NOMINMAX
#include <Windows.h>
#define STB_IMAGE_IMPLEMENTATION
#define STBI_ONLY_PNG
#include "stb_image.h"

image::image(vec2i size) :
	_borrowed_pointer(false),
	_data(new color[size.size()]),
	_bounds(vec2i::zero, size)
{
}

image::image(vec2i size, color* data) :
	_borrowed_pointer(true),
	_data(data),
	_bounds(vec2i::zero, size) {
}

image::image(std::string path) : _borrowed_pointer(false) {
	int w, h, n;
	_data = reinterpret_cast<color*>(stbi_load(path.c_str(), &w, &h, &n, 0));
	for(int i = 0; i < w*h; ++i) {
		_data[i] = _data[i] & 0xff000000 | (_data[i] & 0x00ff0000) >> 16 | (_data[i] & 0x0000ff00) | (_data[i] & 0x000000ff) << 16;
	}
	_bounds = recti(vec2i::zero, vec2i(w, h));
}

image::~image() {
	if(!_borrowed_pointer)
		delete _data;
}

vec2i const& image::size() const {
	return _bounds.size;
}

recti const& image::bounds() const {
	return _bounds;
}

color* image::raw_pointer() const {
	return _data;
}

void image::clear(color color) {
	std::fill(_data, _data + size().size() - 1, color);
}



void image::draw(image const& image, vec2i pos) {
	auto crop = recti::intersection(bounds(), recti(pos, image.bounds().size));
	auto src_off = crop.pos - pos;
	auto start = crop.pos;
	auto end = crop.pos2();
	for(int y = start.y; y <= end.y; ++y) {
		for(int x = start.x; x <= end.x; ++x) {
			auto color = image._data[x - start.x + src_off.x + (y - start.y + src_off.y) * image._bounds.size.x];
			if(color & 0xff000000)
				_data[x + y * _bounds.size.x] = color;
		}
	}
}

void image::draw(image const& image, vec2i pos, recti src_rect) {
}

void image::draw_upscaled(image const& image) {
	auto ratio = _bounds.size / image._bounds.size;
	auto scale = std::min(ratio.x, ratio.y);
	switch(scale) {
	case 3:
#pragma omp parallel for
		for(int y = 0; y < image._bounds.size.y; ++y) {
			for(int x = 0; x < image._bounds.size.x; ++x) {
				auto color = image._data[x + y * image._bounds.size.x];
				auto color2 = (color & 0xfefefe) >> 1;
				_data[x * 3 + 0 + (y * 3 + 0) * _bounds.size.x] = color;
				_data[x * 3 + 1 + (y * 3 + 0) * _bounds.size.x] = color;
				_data[x * 3 + 2 + (y * 3 + 0) * _bounds.size.x] = color2;
				_data[x * 3 + 0 + (y * 3 + 1) * _bounds.size.x] = color;
				_data[x * 3 + 1 + (y * 3 + 1) * _bounds.size.x] = color;
				_data[x * 3 + 2 + (y * 3 + 1) * _bounds.size.x] = color2;
				_data[x * 3 + 0 + (y * 3 + 2) * _bounds.size.x] = color2;
				_data[x * 3 + 1 + (y * 3 + 2) * _bounds.size.x] = color2;
				_data[x * 3 + 2 + (y * 3 + 2) * _bounds.size.x] = color2;
			}
		}
		break;
	default:
#pragma omp parallel for
		for(int y = 0; y < image._bounds.size.y; ++y) {
			for(int x = 0; x < image._bounds.size.x; ++x) {
				auto color = image._data[x + y * image._bounds.size.x];
				for(int sy = 0; sy < scale; ++sy) {
					for(int sx = 0; sx < scale; ++sx) {
						_data[(x * scale + sx) + (y * scale + sy) * _bounds.size.x] = color;
					}
				}
			}
		}
	}
}
